;; Fire-soil-vegetation feedback model to accompany Bowman & Perry in Plant and Soil.
;; The model requires NetLogo v 6.0.1
extensions [palette]

globals [
  fire-front
  fire-size
  n-fires
  ignite-nhb

  ;; i
  rng-seed
  n-patches
  min-forest

  age-flamm-incr

  fire-record
  mean-edge
]

patches-own
[
  state              ;; 1 = grass 2 = tree

  time-forest
  time-grass
  dynamism

  flamm
  tree-colonised?

  time-since-fire
  time-since-colonised
  infertility
  n-boundary
  ;soil
]

;; Setup the world and the savanna-forest interface
to setup
  clear-all

  set fire-record []

  ifelse user-seed?
  [
    set rng-seed u-seed
    random-seed read-from-string rng-seed
  ]
  [
   set rng-seed new-seed
   random-seed rng-seed
  ]

  set n-patches world-width * world-height
  ask patches
  [
    set n-boundary 0
    ifelse pxcor < 0.5 * world-width
    [
       ;set soil
       set state 1
       set tree-colonised? false
       set time-grass 1
       set time-forest 0
    ]
    [
       ;set soil 1
       set state 2
       set tree-colonised? true
       set time-grass 0
       set time-forest 1
    ]

    set age-flamm-incr (1 / (critical-age)) * (grass-flamm - tree-flamm)
  ]


  ask patches
  [
    if state = 2
    [
      set flamm tree-flamm
      set tree-colonised? true
    ]

    if state = 1
    [
      set flamm grass-flamm
    ]
  ]

  let w max-pycor / 2
  ;;ask patches [ set infertility 0.5 + ((pxcor - w) / (w * 2)) ]

  ask patches [ifelse state = 1 [set infertility grass-infertility] [set infertility 1] ]

  colour-patches

  set min-forest count patches with [state = 2]

  reset-ticks

end


;; Each tick check if there is a fire, test for cell state change, write data, etc.
to go

  if n-fires >= no-fires [
    ask patches [set dynamism (min list time-forest time-grass) / ticks]
    if write-record? [
      export-plot "Edge Width" "width.csv"
      export-plot "Mean Edge" "mean.csv"
    ]
    stop
  ]

  ;; Increment age in each patches
  ask patches
  [
    set time-since-fire time-since-fire + 1

    ;;  Forest cells disperse propagules and update fertility if appropriate
    if state = 2
    [
      set time-forest time-forest + 1
      let target patch-at-heading-and-distance (random-float 360) (random-exponential dispersal-distance)
      if target != nobody
      [
         ask target [
           if state = 1 and tree-colonised? = false and random-float 1 < forest-invasion
           [
             set tree-colonised? true
             set time-since-colonised 0
            ]
          ]
      ]
      if dynamic-fertility? [set infertility max (list 1 (infertility - 0.01))]
    ]

    ;; Grassland cells age and test for state change (if colonised by individual older than critical-age * infertility)
    if state = 1
    [
      set time-grass time-grass + 1
      ifelse tree-colonised?
      [
         set time-since-colonised time-since-colonised + 1
         ifelse time-since-colonised >= (critical-age * infertility)
         [
           set state 2
           set flamm tree-flamm
         ]
         [
           set flamm grass-flamm ;; max (list (grass-flamm - (age-flamm-incr * time-since-colonised)) tree-flamm)
         ]
       ]
       [
         set flamm grass-flamm
       ]
    ]
  ]

  colour-patches


  ;; Once the critical-age num,ber of ticks has elapsed (burn in) then fires can occur
  ;; at frequency fire-freq.
  if ticks > critical-age ;; default critical-age
  [
    if random-float 1 <= fire-freq
    [
      set fire-size 0
      igniteFire
      if fire-size != 0 [ fireSpread ]
      post-fire
    ]
  ]

  if count patches with [state = 2] <= min-forest
     [set min-forest count patches with [state = 2]]

  ask patches with [state = 2] with [count neighbors with [state = 1] > 0]
  [
    set n-boundary n-boundary + 1]

  if landscape-plots?
  [
    if ticks = 0 [ snap-landscape ]
    if n-fires mod 10 = 0 and n-fires > 0 [ snap-landscape ]
  ]

  tick

end

;; Start the fire at a random site in the grassland if there is any (otherwise random forest)
to igniteFire
  set fire-size 1
  set n-fires n-fires + 1

  let ignite-site nobody

  ifelse any? patches with [state = 1]
  [
    set ignite-site one-of patches with [state = 1]
    set ignite-nhb count [neighbors with [pcolor = brown]] of ignite-site
  ]
  [
    set ignite-site one-of patches
    set ignite-nhb count [neighbors with [pcolor = brown]] of ignite-site
  ]

  ask ignite-site
  [
     set pcolor red
     set state 0
     set fire-front patch-set self               ;; a new fire-front patch-set
     set fire-size fire-size + 1
  ]
end

;; Fire spreads following a percolation algortih (as per O'Sullivan & Perry 2013)
to fireSpread
  while [ any? fire-front ]                 ;; Stop when we run out of flaming fire-front
  [
    set fire-size fire-size + count fire-front
    fireShell
  ]
end

; Individual percolation shell - uses burning cell agentset to speed things up
to fireShell
   let new-fire-front patch-set nobody     ;; Empty set of patches for the next 'round' of the fire

   ask fire-front [
     ask neighbors with [pcolor != red]
     [
       if random-float 1 <= flamm
       [
         set new-fire-front ( patch-set new-fire-front self)
     ]
     ]
   ]

   ask new-fire-front [set pcolor red set state 0]
   set fire-front new-fire-front
end

;; Change state, etc. in burned cells
to post-fire
  ask patches with [state = 0] [
    set state 1
    set pcolor brown
    set tree-colonised? false
    set time-since-fire 0
    set time-since-colonised 0
    if dynamic-fertility? [set infertility max list 0 (infertility + 0.2)]
  ]

  let fs (sentence ignite-nhb (count patches with [pcolor = green]) fire-size)
  set fire-record lput fs fire-record
end


to colour-patches
  ask patches
  [
    ifelse display-variable = "vegetation"
    [

      if state = 1 and not tree-colonised? [ set pcolor brown ]
      if state = 1 and tree-colonised? [ set pcolor orange ]
      if state = 2 [ set pcolor green ]
    ]
    [

      set pcolor scale-color red flamm (1.5 * grass-flamm) (0.5 * tree-flamm)
    ]
  ]
end

;; Write fire-record to text file if required
to write-record
  let n-run behaviorspace-run-number
  carefully
  [
    file-open "firehistory.txt"

    if n-run = 0 [file-print "run, nhb, p.for, size"]
    foreach fire-record
    [ ?1 ->
      let s ?1
      file-write n-run file-write ","
      foreach s
      [ ??1 ->
        file-write ??1 file-write ","
      ]
      file-print ""
    ]
  file-close
  ]
  [
    show "File write failed."
    print error-message
  ]

end

;; Following three functions all compute location and size of grassland-forest interface
to-report edge-location
  let y-coords n-values max-pycor [ ?1 -> ?1 ]
  let m []

  foreach y-coords
  [ ?1 ->
    let row patches with [pycor = ?1]
    let x min-one-of (row with [state = 2]) [pxcor]

    set m lput ([pxcor] of x) m
  ]

  report mean m
end

to-report edge-location-var
  let y-coords n-values max-pycor [ ?1 -> ?1 ]
  let m []

  foreach y-coords
  [ ?1 ->
    let row patches with [pycor = ?1]
    let x min-one-of (row with [state = 2]) [pxcor]

    set m lput ([pxcor] of x) m
  ]

  report variance m
end

to-report edge-width
  let y-coords n-values max-pycor [ ?1 -> ?1 ]
  let m []

  foreach y-coords
  [ ?1 ->
    let row patches with [pycor = ?1]
    ifelse any? row with [n-boundary > 0]
    [
      let xmin min-one-of (row with [n-boundary > 0]) [pxcor]
      let xmax max-one-of (row with [n-boundary > 0]) [pxcor]

      set m lput (([pxcor] of xmax - [pxcor] of xmin) + 1) m
    ]
    [
      set m lput 0 m
    ]
  ]
  ;show m
  report mean m

end


to colour-edge
  let max-edge max [n-boundary] of patches / 10

  ask patches
  [
    set pcolor scale-color red (n-boundary / 10) max-edge 0
  ]
end

to snap-landscape
  let file-dump (word "landscape" n-fires ".png")
  export-view file-dump

  colour-edge
  set file-dump (word "edge" n-fires ".png")
  export-view file-dump

end

to colour-edge-ramp


end
@#$#@#$#@
GRAPHICS-WINDOW
205
10
813
169
-1
-1
3.0
1
10
1
1
1
0
0
0
1
0
199
0
49
1
1
1
ticks
30.0

BUTTON
17
71
106
104
setup
setup\n
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

BUTTON
108
71
190
104
go
go
T
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

BUTTON
18
105
81
138
step
go
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

SLIDER
16
157
188
190
tree-flamm
tree-flamm
0
0.2
0.035
.005
1
NIL
HORIZONTAL

SLIDER
17
198
189
231
grass-flamm
grass-flamm
0
1.0
0.35
.01
1
NIL
HORIZONTAL

SLIDER
19
253
191
286
critical-age
critical-age
0
100
15.0
1
1
NIL
HORIZONTAL

SLIDER
19
293
191
326
fire-freq
fire-freq
0
1
0.0
.01
1
NIL
HORIZONTAL

SLIDER
19
343
191
376
forest-invasion
forest-invasion
0
.2
0.05
.005
1
NIL
HORIZONTAL

CHOOSER
37
395
175
440
display-variable
display-variable
"vegetation" "flammability"
0

SLIDER
18
24
190
57
no-fires
no-fires
1
200
100.0
5
1
NIL
HORIZONTAL

MONITOR
84
106
159
151
No. of fires
n-fires
0
1
11

PLOT
413
211
753
370
Mean edge
NIL
NIL
0.0
10.0
0.0
300.0
true
false
"" ""
PENS
"location" 1.0 0 -16777216 true "" "plot edge-location"

MONITOR
772
221
844
258
seed
rng-seed
0
1
9

SWITCH
845
224
962
257
user-seed?
user-seed?
1
1
-1000

INPUTBOX
771
260
942
320
u-seed
1895493178
1
0
String

SWITCH
42
450
171
483
write-record?
write-record?
1
1
-1000

SLIDER
22
495
194
528
dispersal-distance
dispersal-distance
1
10
1.0
1
1
NIL
HORIZONTAL

BUTTON
774
330
871
363
colour-edge
colour-edge
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

PLOT
206
213
406
370
Edge Width
NIL
NIL
0.0
10.0
0.0
10.0
true
false
"" ""
PENS
"width" 1.0 0 -16777216 true "" "plot edge-width"
"var" 1.0 0 -2674135 true "" "plot edge-location-var"

SLIDER
205
378
377
411
grass-infertility
grass-infertility
1
10
5.0
.01
1
NIL
HORIZONTAL

SWITCH
381
378
531
411
dynamic-fertility?
dynamic-fertility?
0
1
-1000

SWITCH
535
378
683
411
landscape-plots?
landscape-plots?
1
1
-1000

BUTTON
774
370
873
403
NIL
colour-patches
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

BUTTON
777
408
875
441
colour-dynamics
ask patches with [dynamism > 0]\n[\n    set pcolor palette:scale-gradient [[0 0 255] [255 0 0]] dynamism -0.05 0.55\n\n]
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

TEXTBOX
210
418
360
446
Control how much less fertile than forest grassland is
11
0.0
1

TEXTBOX
384
417
534
435
Does fertility return in forests?\t\t
11
0.0
1

TEXTBOX
209
175
625
193
Brown = treeless, orange = juvenile tree in grassland, green = forest
11
0.0
1

@#$#@#$#@
## WHAT IS IT?

This is a simple grid-based model of feedback dynamics (soil and fire) at savanna-forest interfaces.


## CREDITS AND REFERENCES
@#$#@#$#@
default
true
0
Polygon -7500403 true true 150 5 40 250 150 205 260 250

airplane
true
0
Polygon -7500403 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7500403 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
false
0
Polygon -7500403 true true 150 285 285 225 285 75 150 135
Polygon -7500403 true true 150 135 15 75 150 15 285 75
Polygon -7500403 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7500403 true true 96 182 108
Circle -7500403 true true 110 127 80
Circle -7500403 true true 110 75 80
Line -7500403 true 150 100 80 30
Line -7500403 true 150 100 220 30

butterfly
true
0
Polygon -7500403 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7500403 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7500403 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7500403 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7500403 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7500403 true true 47 195 58
Circle -7500403 true true 195 195 58

circle
false
0
Circle -7500403 true true 0 0 300

circle 2
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240

cow
false
0
Polygon -7500403 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7500403 true true 73 210 86 251 62 249 48 208
Polygon -7500403 true true 25 114 16 195 9 204 23 213 25 200 39 123

cylinder
false
0
Circle -7500403 true true 0 0 300

dot
false
0
Circle -7500403 true true 90 90 120

face happy
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7500403 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7500403 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7500403 true true 60 15 75 300
Polygon -7500403 true true 90 150 270 90 90 30
Line -7500403 true 75 135 90 135
Line -7500403 true 75 45 90 45

flower
false
0
Polygon -10899396 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7500403 true true 85 132 38
Circle -7500403 true true 130 147 38
Circle -7500403 true true 192 85 38
Circle -7500403 true true 85 40 38
Circle -7500403 true true 177 40 38
Circle -7500403 true true 177 132 38
Circle -7500403 true true 70 85 38
Circle -7500403 true true 130 25 38
Circle -7500403 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -10899396 true false 189 233 219 188 249 173 279 188 234 218
Polygon -10899396 true false 180 255 150 210 105 210 75 240 135 240

house
false
0
Rectangle -7500403 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7500403 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7500403 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7500403 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7500403 true 150 0 150 300

line half
true
0
Line -7500403 true 150 0 150 150

pentagon
false
0
Polygon -7500403 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7500403 true true 110 5 80
Polygon -7500403 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7500403 true true 127 79 172 94
Polygon -7500403 true true 195 90 240 150 225 180 165 105
Polygon -7500403 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7500403 true true 135 90 165 300
Polygon -7500403 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7500403 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7500403 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7500403 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7500403 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7500403 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7500403 true true 135 90 120 45 150 15 180 45 165 90

square
false
0
Rectangle -7500403 true true 30 30 270 270

square 2
false
0
Rectangle -7500403 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7500403 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7500403 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7500403 true true 120 120 60

tree
false
0
Circle -7500403 true true 118 3 94
Rectangle -6459832 true false 120 195 180 300
Circle -7500403 true true 65 21 108
Circle -7500403 true true 116 41 127
Circle -7500403 true true 45 90 120
Circle -7500403 true true 104 74 152

triangle
false
0
Polygon -7500403 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7500403 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7500403 true true 4 45 195 187
Polygon -7500403 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7500403 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7500403 false true 24 174 42
Circle -7500403 false true 144 174 42
Circle -7500403 false true 234 174 42

turtle
true
0
Polygon -10899396 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -10899396 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -10899396 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -10899396 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -10899396 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7500403 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7500403 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7500403 true 150 285 150 15
Line -7500403 true 15 150 285 150
Circle -7500403 true true 120 120 60
Line -7500403 true 216 40 79 269
Line -7500403 true 40 84 269 221
Line -7500403 true 40 216 269 79
Line -7500403 true 84 40 221 269

x
false
0
Polygon -7500403 true true 270 75 225 30 30 225 75 270
Polygon -7500403 true true 30 75 75 30 270 225 225 270
@#$#@#$#@
NetLogo 6.0.1
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
<experiments>
  <experiment name="edge" repetitions="100" runMetricsEveryStep="true">
    <setup>setup</setup>
    <go>go</go>
    <metric>edge-width</metric>
    <metric>edge-location</metric>
    <metric>edge-location-var</metric>
    <enumeratedValueSet variable="fire-freq">
      <value value="0.05"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="grass-fertility">
      <value value="1"/>
      <value value="5"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="dynamic-fertility?">
      <value value="false"/>
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="tree-flamm">
      <value value="0.035"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="user-seed?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="no-fires">
      <value value="100"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="forest-invasion">
      <value value="0.05"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="critical-age">
      <value value="15"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="write-record?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="dispersal-distance">
      <value value="1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="grass-flamm">
      <value value="0.35"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="display-variable">
      <value value="&quot;vegetation&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="u-seed">
      <value value="&quot;1895493178&quot;"/>
    </enumeratedValueSet>
  </experiment>
</experiments>
@#$#@#$#@
@#$#@#$#@
default
0.0
-0.2 0 0.0 1.0
0.0 1 1.0 0.0
0.2 0 0.0 1.0
link direction
true
0
Line -7500403 true 150 150 90 180
Line -7500403 true 150 150 210 180
@#$#@#$#@
0
@#$#@#$#@
